@@ -1,4 +1,4 @@
-(* 
+(*
  * BatIO - Abstract input/output
  * Copyright (C) 2003 Nicolas Cannasse
  *               2008 David Teller (contributor)
@@ -22,7 +22,7 @@
  *)
 
 (** High-order abstract I/O.
-    
+
     This module deals with {!type: input}s and {!type:
     output}s. Inputs are manners of getting information from the
     outside world and into your program (for instance, reading from
@@ -62,8 +62,8 @@
     faster and induce less wear on the hardware. Occasionally, you
     may wish to force all waiting operations to take place {e now}.
     For this purpose, you may either function {!flush} or function
-I    {!flush_out}.
-    
+    I    {!flush_out}.
+
     Once you have finished using your {!type: input} or your {!type:
     output}, chances are that you will want to close it. This is not a
     strict necessity, as OCaml will eventually close it for you when
@@ -99,7 +99,6 @@ I    {!flush_out}.
     @documents BatInnerIO
 *)
 
-open BatCamomile
 open BatInnerIO
 
 type input = BatInnerIO.input
@@ -109,9 +108,15 @@ type 'a output = 'a BatInnerIO.output
 (** The abstract output type, ['a] is the accumulator data, it is returned
 	when the [close_out] function is called. *)
 
+type ('a, 'b) printer = 'b output -> 'a -> unit
+(** The type of a printing function to print a ['a] to an output that
+    produces ['b] as result. *)
+
+type 'a f_printer = Format.formatter -> 'a -> unit
+
 exception No_more_input
 (** This exception is raised when reading on an input with the [read] or
-  [nread] functions while there is no available token to read. *)
+    [nread] functions while there is no available token to read. *)
 
 exception Input_closed
 (** This exception is raised when reading on a closed input. *)
@@ -140,7 +145,7 @@ val stdout: unit output
 
 val stderr: unit output
 (** Standard error output, as per Unix/Windows conventions.
-   
+
     Use this output to display warnings and error messages.
 
     Example: [
@@ -170,47 +175,47 @@ val read : input -> char
 
 val nread : input -> int -> string
 (** [nread i n] reads a string of size up to [n] from an input.
-  The function will raise [No_more_input] if no input is available.
-  It will raise [Invalid_argument] if [n] < 0.
+    The function will raise [No_more_input] if no input is available.
+    It will raise [Invalid_argument] if [n] < 0.
 
     Example: [let read_md5 ch = nread ch 32]
 *)
 
 val really_nread : input -> int -> string
 (** [really_nread i n] reads a string of exactly [n] characters
-  from the input. Raises [No_more_input] if at least [n] characters are
-  not available. Raises [Invalid_argument] if [n] < 0.
+    from the input. @raise No_more_input if at least [n] characters are
+    not available. @raise Invalid_argument if [n] < 0.
 
     Example: [let read_md5 ch = really_nread ch 32]
 *)
 
 val input : input -> string -> int -> int -> int
-  (** [input i s p l] reads up to [l] characters from the given input,
-      storing them in string [s], starting at character number [p]. It
-      returns the actual number of characters read (which may be 0) or
-      raise [No_more_input] if no character can be read. It will raise
-      [Invalid_argument] if [p] and [l] do not designate a valid
-      substring of [s].
+(** [input i s p l] reads up to [l] characters from the given input,
+    storing them in string [s], starting at character number [p]. It
+    returns the actual number of characters read (which may be 0) or
+    raise [No_more_input] if no character can be read. It will raise
+    [Invalid_argument] if [p] and [l] do not designate a valid
+    substring of [s].
 
-      Example: [let map_ch f ?(block_size=100) =
-      let b = String.create block_size in
-      try while true do
-        let l = input ch b 0 block_size in
-        f b 0 l;
-      done with No_more_input -> ()]
-  *)
+    Example: [let map_ch f ?(block_size=100) =
+    let b = String.create block_size in
+    try while true do
+      let l = input ch b 0 block_size in
+      f b 0 l;
+    done with No_more_input -> ()]
+*)
 
 val really_input : input -> string -> int -> int -> int
-  (** [really_input i s p l] reads exactly [l] characters from the
-      given input, storing them in the string [s], starting at
-      position [p]. For consistency with {!BatIO.input} it returns
-      [l]. Raises [No_more_input] if at [l] characters are not
-      available. Raises [Invalid_argument] if [p] and [l] do not
-      designate a valid substring of [s].
+(** [really_input i s p l] reads exactly [l] characters from the
+    given input, storing them in the string [s], starting at
+    position [p]. For consistency with {!BatIO.input} it returns
+    [l]. @raise No_more_input if at [l] characters are not
+    available. @raise Invalid_argument if [p] and [l] do not
+    designate a valid substring of [s].
 
-      Example: [let _ = really_input stdin b 0 3]
+    Example: [let _ = really_input stdin b 0 3]
 
-  *)
+*)
 
 val close_in : input -> unit
 (** Close the input. It can no longer be read from.
@@ -218,28 +223,22 @@ val close_in : input -> unit
     Example: [close_in network_in;]
 *)
 
-val write : 'a output -> char -> unit
+val write : (char, _) printer
 (** Write a single char to an output.
 
     Example: [write stdout 'x';]
 *)
 
-val nwrite : 'a output -> string -> unit
+val nwrite : (string, _) printer
 (** Write a string to an output.
 
     Example: [nwrite stdout "Enter your name: ";]
 *)
 
-val write_buf: 'a output -> Buffer.t -> unit
-(** Write the contents of a buffer to an output.
-
-    Example: [let b = Buffer.create 10 in for i = 1 to 100 do Buffer.add (string_of_int i); Buffer.add " "; done; nwrite stdout b;]
-*)
-
 val output : 'a output -> string -> int -> int -> int
 (** [output o s p l] writes up to [l] characters from string [s], starting at
-  offset [p]. It returns the number of characters written. It will raise
-  [Invalid_argument] if [p] and [l] do not designate a valid substring of [s].
+    offset [p]. It returns the number of characters written. It will raise
+    [Invalid_argument] if [p] and [l] do not designate a valid substring of [s].
 
     Example: [let str = "Foo Bar Baz" in let written = output stdout str 2 4;]
 
@@ -248,9 +247,9 @@ val output : 'a output -> string -> int -> int -> int
 
 val really_output : 'a output -> string -> int -> int -> int
 (** [really_output o s p l] writes exactly [l] characters from string [s] onto
-  the the output, starting with the character at offset [p]. For consistency with
-  {!BatIO.output} it returns [l]. Raises [Invalid_argument] if [p] and [l] do not
-  designate a valid substring of [s].
+    the the output, starting with the character at offset [p]. For consistency with
+    {!BatIO.output} it returns [l]. @raise Invalid_argument if [p] and [l] do not
+    designate a valid substring of [s].
 
     This function is useful for networking situations where the output
     buffer might fill resulting in not the entire substring being
@@ -259,11 +258,11 @@ val really_output : 'a output -> string -> int -> int -> int
 *)
 
 val flush : 'a output -> unit
-  (** Flush an output.
+(** Flush an output.
 
-      If previous write operations have caused errors, this may trigger an exception.
+    If previous write operations have caused errors, this may trigger an exception.
 
-      Example: [flush stdout;]
+    Example: [flush stdout;]
 *)
 
 val flush_all : unit -> unit
@@ -299,9 +298,9 @@ val close_all : unit -> unit
 *)
 (**/**)
 
-(** {6 Creation of BatIO Inputs/Outputs} 
+(** {6 Creation of BatIO Inputs/Outputs}
 
-    To open a file for reading/writing, see {!File.open_file_in}
+    To open a file for reading/writing, see {!File.open_in}
     and {!File.open_file_out}*)
 
 val input_string : string -> input
@@ -318,19 +317,13 @@ val input_string : string -> input
 
 val output_string : unit -> string output
 (** Create an output that will write into a string in an efficient way.
-  When closed, the output returns all the data written into it. *)
-
-val output_buffer : Buffer.t -> string output
-(** Create an output that will append its results at the end of a buffer
-    in an efficient way. Closing  returns the whole contents of the buffer
-    -- the buffer remains usable.*)
-    
+    When closed, the output returns all the data written into it. *)
 
 val input_enum : char BatEnum.t -> input
 (** Create an input that will read from an [enum]. *)
 
 val output_enum : unit -> char BatEnum.t output
-(** Create an output that will write into an [enum]. The 
+(** Create an output that will write into an [enum]. The
     final enum is returned when the output is closed. *)
 
 val combine : ('a output * 'b output) -> ('a * 'b) output
@@ -338,19 +331,19 @@ val combine : ('a output * 'b output) -> ('a * 'b) output
     writing to [c] will actually write to both [a] and [b] *)
 
 val tab_out : ?tab:char -> int -> 'a output -> unit output
-  (** Create an output shifted to the right by a number of white spaces
-      (or [tab], if given).
+(** Create an output shifted to the right by a number of spaces
+    (or other character as specified by [tab]).
 
-      [tab_out n out] produces a new output for writing into [out], in
-      which every new line starts with [n] white spaces.
-      Raises [Invalid_argument] if [n] < 0.
+    [tab_out n out] produces a new output for writing into [out], in
+    which every new line starts with [n] spaces.
+    @raise Invalid_argument if [n] < 0.
 
-      Closing [tab_out n out] does not close [out]. Rather,
-      closing [out] closes [tab_out n out].
-  *)
+    Closing [tab_out n out] does not close [out]. Rather,
+    closing [out] closes [tab_out n out].
+*)
 
 (*val repeat: int -> 'a output -> unit output
-(** [repeat n out] create an output in which every character or string is repeated 
+  (** [repeat n out] create an output in which every character or string is repeated
     [n] times to [out].*)*)
 
 (** {6 Utilities} *)
@@ -358,12 +351,9 @@ val tab_out : ?tab:char -> int -> 'a output -> unit output
 val read_all : input -> string
 (** read all the contents of the input until [No_more_input] is raised. *)
 
-val read_uall : input -> BatRope.t
-(** Read the whole contents of a UTF-8 encoded input*)
-
 val pipe : unit -> input * unit output
 (** Create a pipe between an input and an ouput. Data written from
-    the output can be read from the input. 
+    the output can be read from the input.
 *)
 
 val copy : ?buffer:int -> input -> _ output -> unit
@@ -374,23 +364,23 @@ val copy : ?buffer:int -> input -> _ output -> unit
 *)
 
 val pos_in : input -> input * (unit -> int)
-  (** Create an input that provide a count function of the number of bytes
-      read from it. *)
+(** Create an input that provide a count function of the number of bytes
+    read from it. *)
 
-val progress_in : input -> (unit -> unit) -> input 
-  (** [progress_in inp f] create an input that calls [f ()]
-      whenever some content is succesfully read from it.*)
+val progress_in : input -> (unit -> unit) -> input
+(** [progress_in inp f] create an input that calls [f ()]
+    whenever some content is succesfully read from it.*)
 
 val pos_out : 'a output -> unit output * (unit -> int)
 (** Create an output that provide a count function of the number of bytes
     written through it. *)
 
 val progress_out : 'a output -> (unit -> unit) -> unit output
-  (** [progress_out out f] create an output that calls [f ()]
-      whenever some content is succesfully written to it.*)
+(** [progress_out out f] create an output that calls [f ()]
+    whenever some content is succesfully written to it.*)
 
 external cast_output : 'a output -> unit output = "%identity"
-(** You can safely transform any output to an unit output in a safe way 
+(** You can safely transform any output to an unit output in a safe way
     by using this function. *)
 
 
@@ -419,8 +409,8 @@ val read_i16 : input -> int
 (** Read a signed 16-bit word. *)
 
 val read_i32 : input -> int
-  (** Read a signed 32-bit integer. Raise [Overflow] if the
-      read integer cannot be represented as a Caml 31-bit integer. *)
+(** Read a signed 32-bit integer. @raise Overflow if the
+    read integer cannot be represented as an OCaml 31-bit integer. *)
 
 val read_real_i32 : input -> int32
 (** Read a signed 32-bit integer as an OCaml int32. *)
@@ -434,68 +424,50 @@ val read_float : input -> float
 val read_double : input -> float
 (** Read an IEEE double precision floating point value. *)
 
-val read_uchar: input -> UChar.t
-(** Read one UChar from a UTF-8 encoded input*)
-
 val read_string : input -> string
 (** Read a null-terminated string. *)
 
-val read_rope: input -> int -> BatRope.t
-(** Read up to n uchars from a UTF-8 encoded input*)
-
 val read_line : input -> string
-  (** Read a LF or CRLF terminated string. If the source runs out of
-      input before a LF is found, returns a string of the remaining input.
-      Will raise [No_more_input] only if no characters are available. *)
+(** Read a LF or CRLF terminated string. If the source runs out of
+    input before a LF is found, returns a string of the remaining input.
+    Will raise [No_more_input] only if no characters are available. *)
 
-val read_uline: input -> BatRope.t
-(** Read a line of UTF-8*)
-
-val write_byte : 'a output -> int -> unit
+val write_byte : (int, _) printer
 (** Write an unsigned 8-bit byte. *)
 
-val write_ui16 : 'a output -> int -> unit
+val write_ui16 : (int, _) printer
 (** Write an unsigned 16-bit word. *)
 
-val write_i16 : 'a output -> int -> unit
+val write_i16 : (int, _) printer
 (** Write a signed 16-bit word. *)
 
-val write_i32 : 'a output -> int -> unit
-(** Write a signed 32-bit integer. *) 
+val write_i32 : (int, _) printer
+(** Write a signed 32-bit integer. *)
 
-val write_real_i32 : 'a output -> int32 -> unit
+val write_real_i32 : (int32, _) printer
 (** Write an OCaml int32. *)
 
-val write_i64 : 'a output -> int64 -> unit
+val write_i64 : (int64, _) printer
 (** Write an OCaml int64. *)
 
-val write_double : 'a output -> float -> unit
+val write_double : (float, _) printer
 (** Write an IEEE double precision floating point value. *)
 
-val write_uchar: _ output -> UChar.t -> unit
-(** Write one uchar to a UTF-8 encoded output.*)
-
-val write_float : 'a output -> float -> unit
+val write_float : (float, _) printer
 (** Write an IEEE single precision floating point value. *)
 
-val write_string : 'a output -> string -> unit
+val write_string : (string, _) printer
 (** Write a string and append an null character. *)
 
-val write_rope : _ output -> BatRope.t -> unit
-(** Write a character rope onto a UTF-8 encoded output.*)
-
-val write_line : 'a output -> string -> unit
+val write_line : (string, _) printer
 (** Write a line and append a line end.
-    
+
     This adds the correct line end for your operating system.  That
     is, if you are writing to a file and your system imposes that
     files should end lines with character LF (or ['\n']), as Unix,
     then a LF is inserted at the end of the line. If your system
     favors CRLF (or ['\r\n']), then this is what will be inserted.*)
 
-val write_uline: _ output -> BatRope.t -> unit
-(** Write one line onto a UTF-8 encoded output.*)
-
 (** Same operations as module {!BatIO}, but with big-endian encoding *)
 module BigEndian :
 sig
@@ -506,95 +478,79 @@ sig
       Generally, to use this module you will wish to either open both
       {!BatIO} and {!BigEndian}, so as to import a big-endian version of
       {!BatIO}, as per
-      [open System.BatIO, BigEndian in ...], 
+      [open System.BatIO, BigEndian in ...],
       or to redefine locally {!BatIO} to use big-endian encodings
       [module BatIO = System.BatIO include BigEndian]
 
   *)
 
-	val read_ui16 : input -> int
-	  (** Read an unsigned 16-bit word. *)
+  val read_ui16 : input -> int
+  (** Read an unsigned 16-bit word. *)
 
-	val read_i16 : input -> int
-	  (** Read a signed 16-bit word. *)
+  val read_i16 : input -> int
+  (** Read a signed 16-bit word. *)
 
-	val read_i32 : input -> int
-	  (** Read a signed 32-bit integer. Raise [Overflow] if the
-	      read integer cannot be represented as a Caml 31-bit integer. *)
+  val read_i32 : input -> int
+  (** Read a signed 32-bit integer. @raise Overflow if the
+	      read integer cannot be represented as an OCaml 31-bit integer. *)
 
-	val read_real_i32 : input -> int32
-	  (** Read a signed 32-bit integer as an OCaml int32. *)
+  val read_real_i32 : input -> int32
+  (** Read a signed 32-bit integer as an OCaml int32. *)
 
-	val read_i64 : input -> int64
-	  (** Read a signed 64-bit integer as an OCaml int64. *)
+  val read_i64 : input -> int64
+  (** Read a signed 64-bit integer as an OCaml int64. *)
 
 
-	val read_double : input -> float
-	  (** Read an IEEE double precision floating point value. *)
+  val read_double : input -> float
+  (** Read an IEEE double precision floating point value. *)
 
-	val read_float: input -> float
-	  (** Read an IEEE single precision floating point value. *)
+  val read_float: input -> float
+  (** Read an IEEE single precision floating point value. *)
 
-	val write_ui16 : 'a output -> int -> unit
-	  (** Write an unsigned 16-bit word. *)
+  val write_ui16 : (int, _) printer
+  (** Write an unsigned 16-bit word. *)
 
-	val write_i16 : 'a output -> int -> unit
-	  (** Write a signed 16-bit word. *)
+  val write_i16 : (int, _) printer
+  (** Write a signed 16-bit word. *)
 
-	val write_i32 : 'a output -> int -> unit
-	  (** Write a signed 32-bit integer. *) 
+  val write_i32 : (int, _) printer
+  (** Write a signed 32-bit integer. *)
 
-	val write_real_i32 : 'a output -> int32 -> unit
-	  (** Write an OCaml int32. *)
+  val write_real_i32 : (int32, _) printer
+  (** Write an OCaml int32. *)
 
-	val write_i64 : 'a output -> int64 -> unit
-	  (** Write an OCaml int64. *)
+  val write_i64 : (int64, _) printer
+  (** Write an OCaml int64. *)
 
-	val write_double : 'a output -> float -> unit
-	  (** Write an IEEE double precision floating point value. *)
+  val write_double : (float, _) printer
+  (** Write an IEEE double precision floating point value. *)
 
-	val write_float  : 'a output -> float -> unit
-	  (** Write an IEEE single precision floating point value. *)
+  val write_float  : (float, _) printer
+  (** Write an IEEE single precision floating point value. *)
 
-	val ui16s_of : input -> int BatEnum.t
-	  (** Read an enumeration of unsigned 16-bit words. *)
+  val ui16s_of : input -> int BatEnum.t
+  (** Read an enumeration of unsigned 16-bit words. *)
 
-	val i16s_of : input -> int BatEnum.t
-	  (** Read an enumartion of signed 16-bit words. *)
+  val i16s_of : input -> int BatEnum.t
+  (** Read an enumartion of signed 16-bit words. *)
 
-	val i32s_of : input -> int BatEnum.t
-	  (** Read an enumeration of signed 32-bit integers. Raise [Overflow] if the
-	      read integer cannot be represented as a Caml 31-bit integer. *)
+  val i32s_of : input -> int BatEnum.t
+  (** Read an enumeration of signed 32-bit integers.
 
-	val real_i32s_of : input -> int32 BatEnum.t
-	  (** Read an enumeration of signed 32-bit integers as OCaml [int32]s. *)
+	    @raise Overflow if the read integer cannot be represented as an OCaml
+	    31-bit integer. *)
 
-	val i64s_of : input -> int64 BatEnum.t
-	  (** Read an enumeration of signed 64-bit integers as OCaml [int64]s. *)
+  val real_i32s_of : input -> int32 BatEnum.t
+  (** Read an enumeration of signed 32-bit integers as OCaml [int32]s. *)
 
-	val doubles_of : input -> float BatEnum.t
-	  (** Read an enumeration of IEEE double precision floating point values. *)
+  val i64s_of : input -> int64 BatEnum.t
+  (** Read an enumeration of signed 64-bit integers as OCaml [int64]s. *)
 
-	val write_bytes : 'a output -> int BatEnum.t -> unit
-	  (** Write an enumeration of unsigned 8-bit bytes. *)
+  val doubles_of : input -> float BatEnum.t
+  (** Read an enumeration of IEEE double precision floating point values. *)
 
-	val write_ui16s : 'a output -> int BatEnum.t -> unit
-	  (** Write an enumeration of unsigned 16-bit words. *)
-
-	val write_i16s : 'a output -> int BatEnum.t -> unit
-	  (** Write an enumeration of signed 16-bit words. *)
-
-	val write_i32s : 'a output -> int BatEnum.t -> unit
-	  (** Write an enumeration of signed 32-bit integers. *) 
-
-	val write_real_i32s : 'a output -> int32 BatEnum.t -> unit
-	  (** Write an enumeration of OCaml int32s. *)
-
-	val write_i64s : 'a output -> int64 BatEnum.t -> unit
-	  (** Write an enumeration of OCaml int64s. *)
-
-	val write_doubles : 'a output -> float BatEnum.t -> unit
-	  (** Write an enumeration of IEEE double precision floating point value. *)
+  val floats_of : input -> float BatEnum.t
+      (** Read an enumeration of IEEE single precision floating point values. *)
 
 end
 
@@ -621,7 +577,7 @@ val read_bits : in_bits -> int -> int
 
 val write_bits : out_bits -> nbits:int -> int -> unit
 (** Write up to 31 bits represented as a value, raise Bits_error if nbits < 0
- or nbits > 31 or the value representation excess nbits. *)
+    or nbits > 31 or the value representation excess nbits. *)
 
 val flush_bits : out_bits -> unit
 (** Flush remaining unwritten bits, adding up to 7 bits which values 0. *)
@@ -637,9 +593,9 @@ val drop_bits : in_bits -> unit
 
 val create_in :
   read:(unit -> char) ->
-  input:(string -> int -> int -> int) -> 
+  input:(string -> int -> int -> int) ->
   close:(unit -> unit) -> input
-(** Fully create an input by giving all the needed functions. 
+(** Fully create an input by giving all the needed functions.
 
     {b Note} Do {e not} use this function for creating an input
     which reads from one or more underlying inputs. Rather, use
@@ -648,45 +604,45 @@ val create_in :
 
 val wrap_in :
   read:(unit -> char) ->
-  input:(string -> int -> int -> int) -> 
-  close:(unit -> unit) -> 
+  input:(string -> int -> int -> int) ->
+  close:(unit -> unit) ->
   underlying:(input list) ->
   input
-(** Fully create an input reading from other inputs by giving all 
-    the needed functions. 
+(** Fully create an input reading from other inputs by giving all
+    the needed functions.
 
     This function is a more general version of {!create_in}
     which also handles dependency management between inputs.
 
     {b Note} When you create an input which reads from another
-    input, function [close] should {e not} close the inputs of 
+    input, function [close] should {e not} close the inputs of
     [underlying]. Doing so is a common error, which could result
     in inadvertently closing {!stdin} or a network socket, etc.
 *)
 
 val inherit_in:
   ?read:(unit -> char) ->
-  ?input:(string -> int -> int -> int) -> 
-  ?close:(unit -> unit) -> 
+  ?input:(string -> int -> int -> int) ->
+  ?close:(unit -> unit) ->
   input -> input
-  (** Simplified and optimized version of {!wrap_in} which may be used
-      whenever only one input appears as dependency.
+(** Simplified and optimized version of {!wrap_in} which may be used
+    whenever only one input appears as dependency.
 
-      [inherit_in inp] will return an input identical to [inp].
-      [inherit_in ~read inp] will return an input identical to
-      [inp] except for method [read], etc.
+    [inherit_in inp] will return an input identical to [inp].
+    [inherit_in ~read inp] will return an input identical to
+    [inp] except for method [read], etc.
 
-      You do not need to close [inp] in [close].
-  *)
+    You do not need to close [inp] in [close].
+*)
 
 
 val create_out :
   write:(char -> unit) ->
-  output:(string -> int -> int -> int) ->   
-  flush:(unit -> unit) -> 
-  close:(unit -> 'a) -> 
+  output:(string -> int -> int -> int) ->
+  flush:(unit -> unit) ->
+  close:(unit -> 'a) ->
   'a output
-    (** 
+(**
 	Fully create an output by giving all the needed functions.
 
 	@param write  Write one character to the output (see {!write}).
@@ -697,14 +653,14 @@ val create_out :
 
 	{b Note} Do {e not} use this function for creating an output which
 	writes to one or more underlying outputs. Rather, use {!wrap_out}.
-    *)
+*)
 
 val wrap_out :
   write:(char -> unit)         ->
-  output:(string -> int -> int -> int) ->   
-  flush:(unit -> unit)         -> 
-  close:(unit -> 'a)           -> 
-  underlying:('b output list)  -> 
+  output:(string -> int -> int -> int) ->
+  flush:(unit -> unit)         ->
+  close:(unit -> 'a)           ->
+  underlying:('b output list)  ->
   'a output
 (**
    Fully create an output that writes to one or more underlying outputs.
@@ -752,9 +708,9 @@ val wrap_out :
 
 val inherit_out:
   ?write:(char -> unit) ->
-  ?output:(string -> int -> int -> int) -> 
+  ?output:(string -> int -> int -> int) ->
   ?flush:(unit -> unit) ->
-  ?close:(unit -> unit) -> 
+  ?close:(unit -> unit) ->
   'a output -> unit output
 (**
    Simplified and optimized version of {!wrap_out} whenever only
@@ -772,7 +728,7 @@ val inherit_out:
 *)
 
 val input_channel : ?autoclose:bool -> ?cleanup:bool -> in_channel -> input
-(** Create an input that will read from a channel. 
+(** Create an input that will read from a channel.
 
     @param autoclose If true or unspecified, the {!type: input}
     will be automatically closed when the underlying [in_channel]
@@ -784,12 +740,12 @@ val input_channel : ?autoclose:bool -> ?cleanup:bool -> in_channel -> input
 *)
 
 val output_channel : ?cleanup:bool -> out_channel -> unit output
-(** Create an output that will write into a channel. 
+(** Create an output that will write into a channel.
 
     @param cleanup If true, the channel
     will be automatically closed when the {!type: output} is closed.
     Otherwise, you will need to close the channel manually.
-*) 
+*)
 
 
 val to_input_channel : input -> in_channel
@@ -809,7 +765,7 @@ val to_input_channel : input -> in_channel
     [poly_in_channel] and/or [poly_out_channel] which are the common
     BatIO specifications established for ExtLib, OCamlNet and
     Camomile.
-    
+
     (see http://www.ocaml-programming.de/tmp/BatIO-Classes.html for more details).
 
     {b Note} In this version of Batteries Included, the object wrappers are {e not}
@@ -818,28 +774,28 @@ val to_input_channel : input -> in_channel
 
 class in_channel : input ->
   object
-	method input : string -> int -> int -> int
-	method close_in : unit -> unit
+    method input : string -> int -> int -> int
+    method close_in : unit -> unit
   end
 
 class out_channel : 'a output ->
   object
-	method output : string -> int -> int -> int
-	method flush : unit -> unit
-	method close_out : unit -> unit
+    method output : string -> int -> int -> int
+    method flush : unit -> unit
+    method close_out : unit -> unit
   end
 
 class in_chars : input ->
   object
-	method get : unit -> char
-	method close_in : unit -> unit
+    method get : unit -> char
+    method close_in : unit -> unit
   end
 
 class out_chars : 'a output ->
   object
-	method put : char -> unit
-	method flush : unit -> unit
-	method close_out : unit -> unit
+    method put : char -> unit
+    method flush : unit -> unit
+    method close_out : unit -> unit
   end
 
 val from_in_channel : #in_channel -> input
@@ -862,8 +818,8 @@ val i16s_of : input -> int BatEnum.t
 (** Read an enumartion of signed 16-bit words. *)
 
 val i32s_of : input -> int BatEnum.t
-(** Read an enumeration of signed 32-bit integers. Raise [Overflow] if the
-  read integer cannot be represented as a Caml 31-bit integer. *)
+(** Read an enumeration of signed 32-bit integers. @raise Overflow if the
+    read integer cannot be represented as an OCaml 31-bit integer. *)
 
 val real_i32s_of : input -> int32 BatEnum.t
 (** Read an enumeration of signed 32-bit integers as OCaml [int32]s. *)
@@ -874,86 +830,30 @@ val i64s_of : input -> int64 BatEnum.t
 val doubles_of : input -> float BatEnum.t
 (** Read an enumeration of IEEE double precision floating point values. *)
 
+val floats_of : input -> float BatEnum.t
+(** Read an enumeration of IEEE single precision floating point values. *)
+
 val strings_of : input -> string BatEnum.t
 (** Read an enumeration of null-terminated strings. *)
 
 val lines_of : input -> string BatEnum.t
 (** Read an enumeration of LF or CRLF terminated strings. *)
- 
-val chunks_of : int -> input -> string BatEnum.t
-(** Read an input as an enumeration of strings of given maximal length.*)
+val lines_of2 : input -> string BatEnum.t
 
-val ulines_of : input -> BatRope.t BatEnum.t
-(** offer the lines of a UTF-8 encoded input as an enumeration*)
+val chunks_of : int -> input -> string BatEnum.t
+(** Read an input as an enumeration of strings of given length.  If the input isn't a multiple of that length, the final string will be smaller than the rest. *)
 
 val chars_of : input -> char BatEnum.t
-(** Read an enumeration of Latin-1 characters. 
+(** Read an enumeration of Latin-1 characters.
 
     {b Note} Usually faster than calling [read] several times.*)
 
-val uchars_of : input -> UChar.t BatEnum.t
-(** offer the characters of an UTF-8 encoded input as an enumeration*)
-
 val bits_of : in_bits -> int BatEnum.t
 (** Read an enumeration of bits *)
 
-val write_bytes : 'a output -> int BatEnum.t -> unit
-(** Write an enumeration of unsigned 8-bit bytes. *)
-
-val write_chars : 'a output -> char BatEnum.t -> unit
-(** Write an enumeration of chars. *)
-
-val write_uchars : _ output -> UChar.t BatEnum.t -> unit
-(** Write an enumeration of characters onto a UTF-8 encoded output.*)
-
-val write_ui16s : 'a output -> int BatEnum.t -> unit
-(** Write an enumeration of unsigned 16-bit words. *)
-
-val write_i16s : 'a output -> int BatEnum.t -> unit
-(** Write an enumeration of signed 16-bit words. *)
-
-val write_i32s : 'a output -> int BatEnum.t -> unit
-(** Write an enumeration of signed 32-bit integers. *) 
-
-val write_real_i32s : 'a output -> int32 BatEnum.t -> unit
-(** Write an enumeration of OCaml int32s. *)
-
-val write_i64s : 'a output -> int64 BatEnum.t -> unit
-(** Write an enumeration of OCaml int64s. *)
-
-val write_doubles : 'a output -> float BatEnum.t -> unit
-(** Write an enumeration of IEEE double precision floating point value. *)
-
-val write_strings : 'a output -> string BatEnum.t -> unit
-(** Write an enumeration of strings, appending null characters.*)
-
-val write_chunks: 'a output -> string BatEnum.t -> unit
-(** Write an enumeration of strings, without appending null characters.*)
-
-val write_lines : 'a output -> string BatEnum.t -> unit
-(** Write an enumeration of lines, appending a LF (it might be converted
-    to CRLF on some systems depending on the underlying BatIO). *)
-
-val write_ropes : 'a output -> BatRope.t BatEnum.t -> unit
-(** Write an enumeration of ropes onto a UTF-8 encoded output,
-    without appending a line-end.*)
-
-val write_ulines : _ output -> BatRope.t BatEnum.t -> unit
-(** Write an enumeration of lines onto a UTF-8 encoded output.*)
-
 val write_bitss : nbits:int -> out_bits -> int BatEnum.t -> unit
 (** Write an enumeration of bits*)
 
-(** {6 Printing} *)
-
-
-val printf : 'a output -> ('b, 'a output, unit) format -> 'b
-(** A [fprintf]-style unparser. For more information
-    about printing, see the documentation of {!Printf}.
-
-    @obsolete Prefer {!Languages.Printf.fprintf}*)
-
-
 val default_buffer_size : int
 (**The default size for internal buffers.*)
 
@@ -1004,23 +904,93 @@ val lock: BatConcurrent.lock ref
 *)
 
 val lock_factory: (unit -> BatConcurrent.lock) ref
-  (**
-     A factory used to create locks. This is used transparently by {!synchronize_in}
-     and {!synchronize_out}.
+(**
+   A factory used to create locks. This is used transparently by {!synchronize_in}
+   and {!synchronize_out}.
 
-     By default, this always returns {!BatConcurrent.nolock}. However,
-     if you're using a version of Batteries compiled in threaded mode,
-     this uses {!BatMutex}.  *)
+   By default, this always returns {!BatConcurrent.nolock}. However,
+   if you're using a version of Batteries compiled in threaded mode,
+   this uses {!BatMutex}.  *)
 
-val to_string : (string output -> 'a -> unit) -> 'a -> string
+val to_string : ('a, string) printer -> 'a -> string
+
+val to_f_printer: ('a, _) printer -> 'a f_printer
 
 (**/**)
 val comb : ('a output * 'a output) -> 'a output
 (** Old name of [combine]*)
 
+val make_enum : (input -> 'a) -> input -> 'a BatEnum.t
+
 (**
    {6 Debugging facilities}
 *)
 
 val get_output_id : _ output -> int
 val get_input_id  : input -> int
+
+module Incubator : sig
+  (** {6 Format-based pretty-printing} *)
+
+  module Array : sig
+    val pp :
+      ?flush:bool ->
+      ?first:string ->
+      ?last:string ->
+      ?sep:string ->
+      ?indent:int ->
+      (Format.formatter -> 'a -> 'b) -> Format.formatter -> 'a array -> unit
+      (** Print the contents of an array, with [first] preceeding the first item
+          (default: ["\[|"]), [last] following the last item (default: ["|\]"])
+          and [sep] separating items (default: ["; "]). A printing function must
+          be provided to print the items in the array. The [flush] parameter
+          (default: [false]) should be set to [true] for the outer-most printing
+          call.  Setting inner calls to [true] - for example, for nested values -
+          prevent indentation from working properly.
+
+          Example:
+            [pp ~flush:true Format.pp_print_int Format.std_formatter \[|1; 2; 3|\]]
+      *)
+  end
+
+  module Enum : sig
+    val pp :
+      ?flush:bool ->
+      ?first:string ->
+      ?last:string ->
+      ?sep:string ->
+      ?indent:int ->
+      (Format.formatter -> 'a -> 'b) -> Format.formatter -> 'a BatEnum.t -> unit
+      (** Print the contents of an enum, with [first] preceeding the first item
+          (default: [""]), [last] following the last item (default: [""])
+          and [sep] separating items (default: [" "]). A printing function must
+          be provided to print the items in the enum. The [flush] parameter
+          (default: [false]) should be set to [true] for the outer-most printing
+          call.  Setting inner calls to [true] - for example, for nested values -
+          prevent indentation from working properly.
+
+          Example:
+            [pp ~flush:true Format.pp_print_int Format.std_formatter (1 -- 3)] *)
+  end
+
+  module List : sig
+    val pp :
+      ?flush:bool ->
+      ?first:string ->
+      ?last:string ->
+      ?sep:string ->
+      ?indent:int ->
+      (Format.formatter -> 'a -> 'b) -> Format.formatter -> 'a list -> unit
+      (** Print the contents of a list, with [first] preceeding the first item
+          (default: ["\["]), [last] following the last item (default: ["\]"])
+          and [sep] separating items (default: ["; "]). A printing function must
+          be provided to print the items in the list. The [flush] parameter
+          (default: [false]) should be set to [true] for the outer-most printing
+          call.  Setting inner calls to [true] - for example, for nested values -
+          prevent indentation from working properly.
+
+          Example:
+            [pp ~flush:true Format.pp_print_int Format.std_formatter \[1; 2; 3\]]
+      *)
+  end
+end
