@@ -59,7 +59,14 @@ type 'a mappable = 'a t
 
 external inj : 'a mut_list -> 'a list = "%identity"
 
-let dummy_node () = { hd = Obj.magic (); tl = [] }
+module Acc = struct
+  let dummy () =
+    { hd = Obj.magic (); tl = [] }
+  let accum acc x =
+    let cell = { hd = x; tl = [] } in
+    acc.tl <- inj cell;
+    cell
+end
 
 let cons h t = h::t
 
@@ -71,7 +78,7 @@ let is_empty = function
 (*$T is_empty
   is_empty []
   not (is_empty [1])
- *)
+*)
 
 let nth l index =
   if index < 0 then invalid_arg "Negative index not allowed";
@@ -88,19 +95,17 @@ let at = nth
   try ignore (at [] 0); false with Invalid_argument _ -> true
   try ignore (at [1;2;3] (-1)); false with Invalid_argument _ -> true
   at [1;2;3] 2 = 3
- *)
+*)
 
 let append l1 l2 =
   match l1 with
   | [] -> l2
   | h :: t ->
     let rec loop dst = function
-    | [] ->
-      dst.tl <- l2
-    | h :: t ->
-      let cell = { hd = h; tl = [] } in
-      dst.tl <- inj cell;
-      loop cell t
+      | [] ->
+        dst.tl <- l2
+      | h :: t ->
+        loop (Acc.accum dst h) t
     in
     let r = { hd = h; tl = [] } in
     loop r t;
@@ -110,15 +115,13 @@ let rec flatten l =
   let rec inner dst = function
     | [] -> dst
     | h :: t ->
-      let r = { hd = h; tl = [] } in
-      dst.tl <- inj r;
-      inner r t
+      inner (Acc.accum dst h) t
   in
   let rec outer dst = function
     | [] -> ()
     | h :: t -> outer (inner dst h) t
   in
-  let r = dummy_node () in
+  let r = Acc.dummy () in
   outer r l;
   r.tl
 
@@ -127,17 +130,20 @@ let concat = flatten
 (*$T flatten
   flatten [[1;2];[3];[];[4;5;6]] = [1;2;3;4;5;6]
   flatten [[]] = []
- *)
+*)
+
+let singleton x = [x]
+(*$Q singleton
+  Q.int (fun x -> let s = singleton x in hd s = x && length s = 1)
+*)
 
 let map f = function
   | [] -> []
   | h :: t ->
     let rec loop dst = function
-    | [] -> ()
-    | h :: t ->
-      let r = { hd = f h; tl = [] } in
-      dst.tl <- inj r;
-      loop r t
+      | [] -> ()
+      | h :: t ->
+        loop (Acc.accum dst (f h)) t
     in
     let r = { hd = f h; tl = [] } in
     loop r t;
@@ -156,18 +162,16 @@ let rec drop n = function
   (drop 3 [1;2;3]) []
   (drop 4 [1;2;3]) []
   (drop 1 [1;2;3]) [2;3]
- *)
+*)
 
 let take n l =
   let rec loop n dst = function
     | h :: t when n > 0 ->
-      let r = { hd = h; tl = [] } in
-      dst.tl <- inj r;
-      loop (n-1) r t
+      loop (n - 1) (Acc.accum dst h) t
     | _ ->
       ()
   in
-  let dummy = dummy_node() in
+  let dummy = Acc.dummy () in
   loop n dummy l;
   dummy.tl
 
@@ -176,17 +180,15 @@ let take n l =
   (take 3 [1;2;3]) [1;2;3]
   (take 4 [1;2;3]) [1;2;3]
   (take 1 [1;2;3]) [1]
- *)
+*)
 
 let take_while p li =
   let rec loop dst = function
     | [] -> ()
     | x :: xs ->
       if p x then
-        let r = { hd = x; tl = [] } in
-        dst.tl <- inj r;
-        loop r xs in
-  let dummy = dummy_node () in
+        loop (Acc.accum dst x) xs in
+  let dummy = Acc.dummy () in
   loop dummy li;
   dummy.tl
 
@@ -196,7 +198,7 @@ let take_while p li =
   (take_while ((=) 3) [4]) []
   (take_while ((=) 3) []) []
   (take_while ((=) 2) [2; 2]) [2; 2]
- *)
+*)
 
 let rec drop_while f = function
   | [] -> []
@@ -206,7 +208,79 @@ let rec drop_while f = function
 (*$= drop_while & ~printer:(IO.to_string (List.print Int.print))
   (drop_while ((=) 3) [3;3;4;3;3]) [4;3;3]
   (drop_while ((=) 3) [3]) []
- *)
+*)
+
+let span p li =
+  let rec loop dst = function
+    | [] -> []
+    | x :: xs as l ->
+      if p x then
+        loop (Acc.accum dst x) xs
+      else l
+  in
+  let dummy = Acc.dummy () in
+  let xs = loop dummy li in
+  (dummy.tl , xs)
+
+(*$= span
+  (span ((=) 3) [3;3;4;3;3])  ([3;3],[4;3;3])
+  (span ((=) 3) [3])          ([3],[])
+  (span ((=) 3) [4])          ([],[4])
+  (span ((=) 3) [])           ([],[])
+  (span ((=) 2) [2; 2])       ([2; 2],[])
+*)
+
+let nsplit p = function
+  | [] -> []
+  (* note that returning [] on empty inputs is an arbitrary choice
+     that is made for consistence with the behavior of
+     BatString.nsplit. Note having this hardcoded case would have
+     `nsplit p []` return `[[]]`, which is also a semantically valid
+     return value (in fact the two are equivalent, but `[[]]` would be
+     a more natural choice as it allows to enforce the simply
+     invariant that `nsplit` return values are always non-empty).
+
+     If that was to redo from scratch, `[[]]` would be a better return
+     value for both `BatList.nsplit` and `BatString.nsplit`.
+  *)
+  | li ->
+    let not_p x = not (p x) in
+    let rec loop dst l =
+      let ok, rest = span not_p l in
+      let r = Acc.accum dst ok in
+      match rest with
+        | [] -> ()
+        | x :: xs -> loop r xs
+    in
+    let dummy = Acc.dummy () in
+    loop dummy li;
+    dummy.tl
+
+(*$T nsplit
+  nsplit ((=) 0) []                    = []
+  nsplit ((=) 0) [0]                   = [[]; []]
+  nsplit ((=) 0) [1; 0]                = [[1]; []]
+  nsplit ((=) 0) [0; 1]                = [[]; [1]]
+  nsplit ((=) 0) [1; 2; 0; 0; 3; 4; 0; 5] = [[1; 2]; []; [3; 4]; [5]]
+*)
+
+let group_consecutive p l =
+  let rec loop dst = function
+    | [] -> ()
+    | x :: rest ->
+      let xs, rest = span (p x) rest in
+      loop (Acc.accum dst (x :: xs)) rest
+  in
+  let dummy = Acc.dummy () in
+  loop dummy l;
+  dummy.tl
+
+(*$= group_consecutive & ~printer:(IO.to_string (List.print (List.print Int.print)))
+  (group_consecutive (=) [3; 3; 4; 3; 3]) [[3; 3]; [4]; [3; 3]]
+  (group_consecutive (=) [3])             [[3]]
+  (group_consecutive (=) [])              []
+  (group_consecutive (=) [2; 2])          [[2; 2]]
+*)
 
 let takewhile = take_while
 let dropwhile = drop_while
@@ -217,18 +291,18 @@ let interleave ?first ?last (sep:'a) (l:'a list) =
     | h::t -> aux (h::sep::acc) t
   in
   match (l,first, last) with
-    | ([],   None,   None)       -> []
-    | ([],   None,   Some x)     -> [x]
-    | ([],   Some x, None)       -> [x]
-    | ([],   Some x, Some y)     -> [x;y]
-    | ([h],  None,   None)       -> [h]
-    | ([h],  None,   Some x)     -> [h;x]
-    | ([h],  Some x, None)       -> [x;h]
-    | ([h],  Some x, Some y)     -> [x;h;y]
-    | (h::t, None  , None )      -> rev (aux [h] t)
-    | (h::t, Some x, None )      -> x::(rev (aux [h] t))
-    | (h::t, None,   Some y)     -> rev_append (aux [h] t) [y]
-    | (h::t, Some x, Some y)     -> x::rev_append (aux [h] t) [y]
+  | ([],   None,   None)       -> []
+  | ([],   None,   Some x)     -> [x]
+  | ([],   Some x, None)       -> [x]
+  | ([],   Some x, Some y)     -> [x;y]
+  | ([h],  None,   None)       -> [h]
+  | ([h],  None,   Some x)     -> [h;x]
+  | ([h],  Some x, None)       -> [x;h]
+  | ([h],  Some x, Some y)     -> [x;h;y]
+  | (h::t, None  , None )      -> rev (aux [h] t)
+  | (h::t, Some x, None )      -> x::(rev (aux [h] t))
+  | (h::t, None,   Some y)     -> rev_append (aux [h] t) [y]
+  | (h::t, Some x, Some y)     -> x::rev_append (aux [h] t) [y]
 
 (*$= interleave & ~printer:(IO.to_string (List.print Int.print))
   (interleave 0 [1;2;3]) [1;0;2;0;3]
@@ -252,11 +326,9 @@ let rec unique ?(eq = ( = )) l =
       match exists (eq h) t with
       | true -> loop dst t
       | false ->
-        let r = { hd =  h; tl = [] }  in
-        dst.tl <- inj r;
-        loop r t
+        loop (Acc.accum dst h) t
   in
-  let dummy = dummy_node() in
+  let dummy = Acc.dummy () in
   loop dummy l;
   dummy.tl
 
@@ -265,7 +337,7 @@ let rec unique ?(eq = ( = )) l =
   [1;2;3;4;5;6] (unique [1;1;2;2;3;3;4;5;6;4;5;6])
   [1] (unique [1;1;1;1;1;1;1;1;1;1])
   [1;2] (unique ~eq:(fun x y -> x land 1 = y land 1) [2;2;2;4;6;8;3;1;2])
- *)
+*)
 
 let unique_cmp ?(cmp = Pervasives.compare) l =
   let set      = ref (BatMap.PMap.create cmp) in
@@ -280,7 +352,7 @@ let unique_cmp ?(cmp = Pervasives.compare) l =
   [1;2;3;4;5;6] (unique_cmp [1;1;2;2;3;3;4;5;6;4;5;6])
   [1] (unique_cmp [1;1;1;1;1;1;1;1;1;1])
   [2;3] (unique_cmp ~cmp:(fun x y -> Int.compare (x land 1) (y land 1)) [2;2;2;4;6;8;3;1;2])
- *)
+*)
 
 
 let unique_hash (type et) ?(hash = Hashtbl.hash) ?(eq = (=)) (l : et list) =
@@ -288,15 +360,15 @@ let unique_hash (type et) ?(hash = Hashtbl.hash) ?(eq = (=)) (l : et list) =
   let ht = HT.create (List.length l) in
   let rec loop dst = function
     | h::t when not (HT.mem ht h) ->
-	HT.add ht h (); (* put h in hash table *)
-	let r = {hd = h; tl = []} in (* and to output list *)
-	dst.tl <- inj r;
-	loop r t
+      HT.add ht h (); (* put h in hash table *)
+      loop
+        (Acc.accum dst h) (* and to output list *)
+        t
     | _::t -> (* if already in hashtable then don't add to output list *)
       loop dst t
     | [] -> ()
   in
-  let dummy = dummy_node() in
+  let dummy = Acc.dummy () in
   loop dummy l;
   dummy.tl
 
@@ -304,7 +376,7 @@ let unique_hash (type et) ?(hash = Hashtbl.hash) ?(eq = (=)) (l : et list) =
   [1;2;3;4;5;6] (unique_hash [1;1;2;2;3;3;4;5;6;4;5;6])
   [1] (unique_hash [1;1;1;1;1;1;1;1;1;1])
   [2;3] (unique_hash ~hash:(fun x -> Hashtbl.hash (x land 1)) ~eq:(fun x y -> x land 1 = y land 1) [2;2;2;4;6;8;3;1;2])
- *)
+*)
 
 let filter_map f l =
   let rec loop dst = function
@@ -313,20 +385,18 @@ let filter_map f l =
       match f h with
       | None -> loop dst t
       | Some x ->
-        let r = { hd = x; tl = [] }  in
-        dst.tl <- inj r;
-        loop r t
+        loop (Acc.accum dst x) t
   in
-  let dummy = dummy_node() in
+  let dummy = Acc.dummy () in
   loop dummy l;
   dummy.tl
 
 let rec find_map f = function
   | [] -> raise Not_found
   | x :: xs ->
-      match f x with
-      | Some y -> y
-      | None -> find_map f xs
+    match f x with
+    | Some y -> y
+    | None -> find_map f xs
 
 let fold_right_max = 1000
 
@@ -348,14 +418,12 @@ let fold_right f l init =
 let map2 f l1 l2 =
   let rec loop dst src1 src2 =
     match src1, src2 with
-      | [], [] -> ()
-      | h1 :: t1, h2 :: t2 ->
-        let r = { hd = f h1 h2; tl = [] } in
-        dst.tl <- inj r;
-        loop r t1 t2
-      | _ -> invalid_arg "map2: Different_list_size"
+    | [], [] -> ()
+    | h1 :: t1, h2 :: t2 ->
+      loop (Acc.accum dst (f h1 h2)) t1 t2
+    | _ -> invalid_arg "map2: Different_list_size"
   in
-  let dummy = dummy_node () in
+  let dummy = Acc.dummy () in
   loop dummy l1 l2;
   dummy.tl
 
@@ -402,9 +470,9 @@ let for_all2 p l1 l2 =
 let exists2 p l1 l2 =
   let rec loop l1 l2 =
     match l1, l2 with
-      | [], [] -> false
-      | h1 :: t1, h2 :: t2 -> if p h1 h2 then true else loop t1 t2
-      | _ -> invalid_arg "exists2: Different_list_size"
+    | [], [] -> false
+    | h1 :: t1, h2 :: t2 -> if p h1 h2 then true else loop t1 t2
+    | _ -> invalid_arg "exists2: Different_list_size"
   in
   loop l1 l2
 
@@ -415,11 +483,9 @@ let remove_assoc x lst =
       if a = x then
         dst.tl <- t
       else
-        let r = { hd = pair; tl = [] } in
-        dst.tl <- inj r;
-        loop r t
+        loop (Acc.accum dst pair) t
   in
-  let dummy = dummy_node () in
+  let dummy = Acc.dummy () in
   loop dummy lst;
   dummy.tl
 
@@ -430,11 +496,9 @@ let remove_assq x lst =
       if a == x then
         dst.tl <- t
       else
-        let r = { hd =  pair; tl = [] } in
-        dst.tl <- inj r;
-        loop r t
+        loop (Acc.accum dst pair) t
   in
-  let dummy = dummy_node() in
+  let dummy = Acc.dummy () in
   loop dummy lst;
   dummy.tl
 
@@ -445,13 +509,11 @@ let find_all p l =
     | [] -> ()
     | h :: t ->
       if p h then
-        let r = { hd = h; tl = [] } in
-        dst.tl <- inj r;
-        findnext r t
+        findnext (Acc.accum dst h) t
       else
         findnext dst t
   in
-  let dummy = dummy_node () in
+  let dummy = Acc.dummy () in
   findnext dummy l;
   dummy.tl
 
@@ -494,6 +556,7 @@ let rec rindex_ofq e l =
 
 let filter = find_all
 
+(* berenger: it is not clear to me if I can use Acc.accum in there *)
 let partition p lst =
   let rec loop yesdst nodst = function
     | [] -> ()
@@ -510,8 +573,8 @@ let partition p lst =
           loop yesdst r t
         end
   in
-  let yesdummy = dummy_node()
-  and nodummy = dummy_node()
+  let yesdummy = Acc.dummy ()
+  and nodummy = Acc.dummy ()
   in
   loop yesdummy nodummy lst;
   yesdummy.tl, nodummy.tl
@@ -520,31 +583,32 @@ let split lst =
   let rec loop adst bdst = function
     | [] -> ()
     | (a, b) :: t ->
-      let x = { hd = a; tl = [] }
-      and y = { hd = b; tl = [] } in
-      adst.tl <- inj x;
-      bdst.tl <- inj y;
-      loop x y t
+      loop (Acc.accum adst a) (Acc.accum bdst b) t
   in
-  let adummy = dummy_node ()
-  and bdummy = dummy_node ()
+  let adummy = Acc.dummy ()
+  and bdummy = Acc.dummy ()
   in
   loop adummy bdummy lst;
   adummy.tl, bdummy.tl
 
 let combine l1 l2 =
-  let rec loop dst l1 l2 =
-    match l1, l2 with
-    | [], [] -> ()
-    | h1 :: t1, h2 :: t2 ->
-      let r = { hd = h1, h2; tl = [] } in
-      dst.tl <- inj r;
-      loop r t1 t2
-    | _, _ -> invalid_arg "combine: Different_list_size"
-  in
-  let dummy = dummy_node () in
-  loop dummy l1 l2;
-  dummy.tl
+  let list_sizes_differ = Invalid_argument "combine: Different_list_size" in
+  match l1, l2 with
+    | [], [] -> []
+    | x :: xs, y :: ys ->
+      let acc = { hd = (x, y); tl = [] } in
+      let rec loop dst l1 l2 = match l1, l2 with
+        | [], [] -> inj acc
+        | h1 :: t1, h2 :: t2 -> loop (Acc.accum dst (h1, h2)) t1 t2
+        | _, _ -> raise list_sizes_differ
+      in loop acc xs ys
+    | _, _ -> raise list_sizes_differ
+
+(*$T combine
+  combine []     []     = []
+  combine [1]    [2]    = [(1, 2)]
+  combine [1; 3] [2; 4] = [(1, 2); (3, 4)]
+*)
 
 let rec init size f =
   if size = 0 then []
@@ -552,9 +616,7 @@ let rec init size f =
   else
     let rec loop dst n =
       if n < size then
-        let r = { hd = f n; tl = [] } in
-        dst.tl <- inj r;
-        loop r (n+1)
+        loop (Acc.accum dst (f n)) (n+1)
     in
     let r = { hd = f 0; tl = [] } in
     loop r 1;
@@ -574,9 +636,7 @@ let mapi f = function
     let rec loop dst n = function
       | [] -> ()
       | h :: t ->
-        let r = { hd = f n h; tl = [] } in
-        dst.tl <- inj r;
-        loop r (n+1) t
+        loop (Acc.accum dst (f n h)) (n + 1) t
     in
     let r = { hd = f 0 h; tl = [] } in
     loop r 1 t;
@@ -606,12 +666,10 @@ let split_nth index = function
     else
       let rec loop n dst l =
         if n = 0 then l else
-        match l with
-        | [] -> invalid_arg "Index past end of list"
-        | h :: t ->
-          let r = { hd =  h; tl = [] } in
-          dst.tl <- inj r;
-          loop (n-1) r t
+          match l with
+          | [] -> invalid_arg "Index past end of list"
+          | h :: t ->
+            loop (n - 1) (Acc.accum dst h) t
       in
       let r = { hd = h; tl = [] } in
       inj r, loop (index-1) r t
@@ -631,11 +689,9 @@ let remove l x =
       if x = h then
         dst.tl <- t
       else
-        let r = { hd = h; tl = [] } in
-        dst.tl <- inj r;
-        loop r t
+        loop (Acc.accum dst h) t
   in
-  let dummy = dummy_node () in
+  let dummy = Acc.dummy () in
   loop dummy l;
   dummy.tl
 
@@ -646,11 +702,9 @@ let rec remove_if f lst =
       if f x then
         dst.tl <- l
       else
-        let r = { hd = x; tl = [] } in
-        dst.tl <- inj r;
-        loop r l
+        loop (Acc.accum dst x) l
   in
-  let dummy = dummy_node () in
+  let dummy = Acc.dummy () in
   loop dummy lst;
   dummy.tl
 
@@ -661,11 +715,9 @@ let rec remove_all l x =
       if x = h then
         loop dst t
       else
-        let r = { hd = h; tl = [] } in
-        dst.tl <- inj r;
-        loop r t
+        loop (Acc.accum dst h) t
   in
-  let dummy = dummy_node () in
+  let dummy = Acc.dummy () in
   loop dummy l;
   dummy.tl
 
@@ -675,8 +727,11 @@ let transpose = function
   | x::xs ->
     let heads = List.map (fun x -> {hd=x; tl=[]}) x in
     ignore ( List.fold_left
-       (fun acc x -> List.map2 (fun x xs -> let r = {hd = x; tl = []} in xs.tl <- inj r; r) x acc)
-       heads xs);
+        (fun acc x ->
+           List.map2
+             (fun x xs -> Acc.accum xs x)
+             x acc)
+        heads xs);
     Obj.magic heads (* equivalent to List.map inj heads, but without creating a new list *)
 
 
@@ -708,11 +763,8 @@ let enum l =
   make (ref l) (ref (-1))
 
 let of_enum e =
-  let h = dummy_node() in
-  let _ = BatEnum.fold (fun acc x ->
-    let r = { hd = x; tl = [] }  in
-    acc.tl <- inj r;
-    r) h e in
+  let h = Acc.dummy () in
+  let _ = BatEnum.fold Acc.accum h e in
   h.tl
 
 
@@ -733,68 +785,135 @@ let of_backwards e =
 
 let assoc_inv e l =
   let rec aux = function
-  | []                  -> raise Not_found
-  | (a,b)::_ when b = e -> a
-  | _::t                -> aux t
+    | []                  -> raise Not_found
+    | (a,b)::_ when b = e -> a
+    | _::t                -> aux t
   in aux l
 
 let assq_inv e l =
   let rec aux = function
-  | []                    -> raise Not_found
-  | (a,b)::_ when b == e  -> a
-  | _::t                  -> aux t
+    | []                    -> raise Not_found
+    | (a,b)::_ when b == e  -> a
+    | _::t                  -> aux t
   in aux l
 
+let modify_opt a f l =
+  let rec aux p = function
+    | [] ->
+      (match f None with
+       | None   -> raise Exit
+       | Some v -> rev ((a,v)::p))
+    | (a',b)::t when a' = a ->
+      (match f (Some b) with
+       | None    -> rev_append p t
+       | Some b' -> rev_append ((a,b')::p) t)
+    | p'::t ->
+      aux (p'::p) t
+  in
+  try aux [] l with Exit -> l
+
+(*$= modify_opt & ~printer:(IO.to_string (List.print (fun fmt (a,b) -> Printf.fprintf fmt "%d,%d" a b)))
+  (* to modify a value *) \
+  (modify_opt 5 (function Some 1 -> Some 2 | _ -> assert false) [ 1,0 ; 5,1 ; 8,2 ]) \
+    [ 1,0 ; 5,2 ; 8,2 ]
+  (* to add a value *) \
+  (modify_opt 5 (function None -> Some 2 | _ -> assert false) [ 1,0 ; 8,2 ]) \
+    [ 1,0 ; 8,2 ; 5,2 ]
+  (* to remove a value *) \
+  (modify_opt 5 (function Some 1 -> None | _ -> assert false) [ 1,0 ; 5,1 ; 8,2 ]) \
+    [ 1,0 ; 8,2 ]
+*)
+
+let modify a f l =
+  let f' = function
+    | None   -> raise Not_found
+    | Some b -> Some (f b)
+  in
+  modify_opt a f' l
+
+(*$= modify & ~printer:(IO.to_string (List.print (fun fmt (a,b) -> Printf.fprintf fmt "%d,%d" a b)))
+  (modify 5 succ [ 1,0 ; 5,1 ; 8,2 ]) [ 1,0 ; 5,2 ; 8,2 ]
+*)
+(*$T modify
+  try ignore (modify 5 succ [ 1,0 ; 8,2 ]); false with Not_found -> true
+*)
+
+let modify_def dfl a f l =
+  let f' = function
+    | None   -> Some (f dfl)
+    | Some b -> Some (f b)
+  in
+  modify_opt a f' l
+
+(*$= modify_def & ~printer:(IO.to_string (List.print (fun fmt (a,b) -> Printf.fprintf fmt "%d,%d" a b)))
+  (modify_def 0 5 succ [ 1,0 ; 5,1 ; 8,2 ]) [ 1,0 ; 5,2 ; 8,2 ]
+  (modify_def 0 5 succ [ 1,0 ; 8,2 ]) [ 1,0 ; 8,2 ; 5,1 ]
+*)
+
 let sort_unique cmp lst =
   let sorted = List.sort cmp lst in
   let fold first rest = List.fold_left
-    (fun (acc, last) elem ->
-       if (cmp last elem) = 0 then (acc, elem)
+      (fun (acc, last) elem ->
+        if (cmp last elem) = 0 then (acc, elem)
         else (elem::acc, elem)
-    )
-    ([first], first)
-    rest
-   in
+      )
+      ([first], first)
+      rest
+  in
   match sorted with
-   | [] -> []
-   | hd::tl ->
-   begin
-    let rev_result, _ = fold hd tl in
-    List.rev rev_result
-   end
+  | [] -> []
+  | hd::tl ->
+    begin
+      let rev_result, _ = fold hd tl in
+      List.rev rev_result
+    end
 
 let group cmp lst =
   let sorted = List.sort cmp lst in
   let fold first rest = List.fold_left
-    (fun (acc, agr, last) elem ->
-       if (cmp last elem) = 0 then (acc, elem::agr, elem)
+      (fun (acc, agr, last) elem ->
+        if (cmp last elem) = 0 then (acc, elem::agr, elem)
         else (agr::acc, [elem], elem)
-    )
-    ([], [first], first)
-    rest
-   in
+      )
+      ([], [first], first)
+      rest
+  in
   match sorted with
-   | [] -> []
-   | hd::tl ->
-   begin
-    let groups, lastgr, _ = fold hd tl in
-    List.rev_map List.rev (lastgr::groups)
-   end
+  | [] -> []
+  | hd::tl ->
+    begin
+      let groups, lastgr, _ = fold hd tl in
+      List.rev_map List.rev (lastgr::groups)
+    end
+
+(*$T group
+  group Pervasives.compare []                 = []
+  group Pervasives.compare [1]                = [[1]]
+  group Pervasives.compare [2; 2]             = [[2; 2]]
+  group Pervasives.compare [5; 4; 4; 2; 1; 6] = [[1]; [2]; [4; 4]; [5]; [6]]
+*)
 
 let cartesian_product l1 l2 =
-   List.concat (List.map (fun i -> List.map (fun j -> (i,j)) l2) l1)
+  List.concat (List.map (fun i -> List.map (fun j -> (i,j)) l2) l1)
 
 (*$T cartesian_product as cp
   cp [1;2;3] ['x';'y'] = [1,'x';1,'y';2,'x';2,'y';3,'x';3,'y']
- *)
+*)
 
-let rec n_cartesian_product = function [] -> assert false
-  | [l] -> List.map (fun i -> [i]) l
+let rec n_cartesian_product = function
+  | [] -> [[]]
   | h :: t ->
-      let rest = n_cartesian_product t in
-      List.concat (List.map (fun i -> List.map (fun r -> i :: r) rest) h)
-
+    let rest = n_cartesian_product t in
+    List.concat (List.map (fun i -> List.map (fun r -> i :: r) rest) h)
 
+(*$T n_cartesian_product as ncp
+  ncp []               = [[]]
+  ncp [[]]             = []
+  ncp [[1]; [2]; [3]]  = [[1;2;3]]
+  ncp [[1;2;3]]        = [[1]; [2]; [3]]
+  ncp [[1;2;3]; []]    = []
+  ncp [[1;2;3]; [4;5]] = [[1;4]; [1;5]; [2;4]; [2;5]; [3;4]; [3;5]]
+*)
 
 let print ?(first="[") ?(last="]") ?(sep="; ") print_a  out = function
   | []   ->
@@ -813,13 +932,52 @@ let print ?(first="[") ?(last="]") ?(sep="; ") print_a  out = function
 let t_printer a_printer _paren out x = print (a_printer false) out x
 
 let reduce f = function [] -> invalid_arg "Empty List"
-  | h::t -> fold_left f h t
+                      | h::t -> fold_left f h t
 
 let min l = reduce Pervasives.min l
 let max l = reduce Pervasives.max l
 let sum l = reduce (+) l
 let fsum l = reduce (+.) l
 
+let min_max ?cmp:(cmp = Pervasives.compare) = function
+  | [] -> invalid_arg "List.min_max: Empty List"
+  | x :: xs ->
+    fold_left
+      (fun (curr_min, curr_max) y ->
+         let new_min =
+           if cmp curr_min y = 1
+           then y
+           else curr_min
+         in
+         let new_max =
+           if cmp curr_max y = -1
+           then y
+           else curr_max
+         in
+         (new_min, new_max)
+      )
+      (x, x)
+      xs
+
+(*$T min_max
+  min_max [1] = (1, 1)
+  min_max [1; 1] = (1, 1)
+  min_max [1; -2; 3; 4; 5; 60; 7; 8] = (-2, 60)
+*)
+
+let unfold b f =
+  let acc = Acc.dummy () in
+  let rec loop dst v =
+    match f v with
+    | None -> acc.tl
+    | Some (a, v) -> loop (Acc.accum dst a) v
+  in loop acc b
+
+(*$T unfold
+  unfold 1 (fun x -> None) = []
+  unfold 0 (fun x -> if x > 3 then None else Some (x, succ x)) = [0;1;2;3]
+*)
+
 module Exceptionless = struct
   let rfind p l =
     try  Some (rfind p l)
@@ -866,9 +1024,9 @@ module Exceptionless = struct
     with Failure "tl" -> None
 
   let rec last = function
-  | [] -> None
-  | [x] -> Some x
-  | _ :: l -> last l
+    | [] -> None
+    | [x] -> Some x
+    | _ :: l -> last l
 end
 
 
@@ -928,27 +1086,27 @@ open BatOrd
 
 let rec eq eq_elt l1 l2 =
   match l1 with
-    | [] -> (match l2 with [] -> true | _ -> false)
-    | hd1::tl1 ->
-      (match l2 with
-	| [] -> false
-	| hd2::tl2 -> bin_eq eq_elt hd1 hd2 (eq eq_elt) tl1 tl2)
+  | [] -> (match l2 with [] -> true | _ -> false)
+  | hd1::tl1 ->
+    (match l2 with
+     | [] -> false
+     | hd2::tl2 -> bin_eq eq_elt hd1 hd2 (eq eq_elt) tl1 tl2)
 
 let rec ord ord_elt l1 l2 =
   match l1 with
-    | [] -> (match l2 with [] -> Eq | _::_ -> Lt)
-    | hd1::tl1 ->
-      (match l2 with
-	| [] -> Gt
-	| hd2::tl2 -> bin_ord ord_elt hd1 hd2 (ord ord_elt) tl1 tl2)
+  | [] -> (match l2 with [] -> Eq | _::_ -> Lt)
+  | hd1::tl1 ->
+    (match l2 with
+     | [] -> Gt
+     | hd2::tl2 -> bin_ord ord_elt hd1 hd2 (ord ord_elt) tl1 tl2)
 
 let rec compare comp_elt l1 l2 =
   match l1 with
-    | [] -> (match l2 with [] -> 0 | _::_ -> -1)
-    | hd1::tl1 ->
-      (match l2 with
-	| [] -> 1
-	| hd2::tl2 -> bin_comp comp_elt hd1 hd2 (compare comp_elt) tl1 tl2)
+  | [] -> (match l2 with [] -> 0 | _::_ -> -1)
+  | hd1::tl1 ->
+    (match l2 with
+     | [] -> 1
+     | hd2::tl2 -> bin_comp comp_elt hd1 hd2 (compare comp_elt) tl1 tl2)
 
 module Eq (T : Eq) = struct
   type t = T.t list
