@@ -146,7 +146,7 @@ let rfind_from str pos sub =
         done;
       done;
       raise Not_found
-  )
+    )
 (*$Q rfind_from
   (Q.triple Q.string Q.char Q.small_int) ~count:1000 (fun (s, c, ofs) -> \
     let v1 = try `res (rfind_from s ofs (String.make 1 c)) with Not_found -> `nf | Invalid_argument _ -> `inv in \
@@ -237,22 +237,22 @@ let tail s pos =
   left "abc" 10 = "abc"
   left "abc" 0 = ""
 *) (*$T right
-  right "abc" 1 = "c"
-  right "ab" 3 = "ab"
-  right "abc" 3 = "abc"
-  right "abc" 0 = ""
-  right "abc" 10 = "abc"
-*) (*$T tail
-  tail "abc" 1 = "bc"
-  tail "ab" 3 = ""
-  tail "abc" 3 = ""
-  tail "abc" 10 = ""
-  tail "abc" 0 = "abc"
-*) (*$T head
-  head "abc" 0 = ""
-  head "abc" 10 = "abc"
-  head "abc" 3 = "abc"
-*)
+     right "abc" 1 = "c"
+     right "ab" 3 = "ab"
+     right "abc" 3 = "abc"
+     right "abc" 0 = ""
+     right "abc" 10 = "abc"
+   *) (*$T tail
+        tail "abc" 1 = "bc"
+        tail "ab" 3 = ""
+        tail "abc" 3 = ""
+        tail "abc" 10 = ""
+        tail "abc" 0 = "abc"
+      *) (*$T head
+        head "abc" 0 = ""
+        head "abc" 10 = "abc"
+        head "abc" 3 = "abc"
+      *)
 
 let split str ~by:sep =
   let p = find str sep in
@@ -316,9 +316,9 @@ let nsplit str ~by:sep =
     aux [] (length str - 1 )
 
 (*$T nsplit
-   nsplit "a;b;c" ~by:";" = ["a"; "b"; "c"]
-   nsplit "" ~by:"x" = []
-   try nsplit "abc" ~by:"" = ["a"; "b"; "c"] with Invalid_argument _ -> true
+  nsplit "a;b;c" ~by:";" = ["a"; "b"; "c"]
+  nsplit "" ~by:"x" = []
+  try nsplit "abc" ~by:"" = ["a"; "b"; "c"] with Invalid_argument _ -> true
   nsplit "a/b/c" ~by:"/" = ["a"; "b"; "c"]
   nsplit "/a/b/c//" ~by:"/" = [""; "a"; "b"; "c"; ""; ""]
   nsplit "FOOaFOObFOOcFOOFOO" ~by:"FOO" = [""; "a"; "b"; "c"; ""; ""]
@@ -442,7 +442,7 @@ let backwards s =
       ~count:(fun () -> !i)
       ~clone:(fun () -> make (BatRef.copy i))
   in
-        make (ref (length s))
+  make (ref (length s))
 (*$T backwards
    "" |> backwards |> of_enum = ""
    "foo" |> backwards |> of_enum = "oof"
@@ -450,7 +450,7 @@ let backwards s =
    for _i = 0 to 2 do BatEnum.junk e done; \
    let e2 = BatEnum.clone e in \
    implode (BatList.of_enum e) = "cba" && implode (BatList.of_enum e2) = "cba"
- *)
+*)
 
 let of_enum e =
   (* TODO: use a buffer when not fast_count *)
@@ -556,7 +556,7 @@ let iteri f str =
     |> Array.to_list
     |> List.filter (fun (_c, pos) -> pos <> [])
   in
- assert_equal ~msg:"String.iteri test"
+  assert_equal ~msg:"String.iteri test"
      (letter_positions "hello")
      ['e',[1]; 'h',[0]; 'l',[2;3]; 'o',[4] ]
 *)
@@ -626,12 +626,18 @@ let replace_chars f s =
 *)
 
 let replace ~str ~sub ~by =
-  try
-    let i = find str sub in
-    (true, (slice ~last:i str) ^ by ^
-      (slice ~first:(i + String.length sub) str))
-  with
-    Not_found -> (false, String.copy str)
+   try
+     let subpos = find str sub in
+     let strlen = length str in
+     let sublen = length sub in
+     let bylen  = length by in
+     let newstr = create (strlen - sublen + bylen) in
+     blit str 0 newstr 0 subpos ;
+     blit by 0 newstr subpos bylen ;
+     blit str (subpos + sublen) newstr (subpos + bylen) (strlen - subpos - sublen) ;
+     (true, newstr)
+   with Not_found ->  (* find failed *)
+     (false, str)
 (*$T replace
    replace ~str:"foobarbaz" ~sub:"bar" ~by:"rab" = (true, "foorabbaz")
    replace ~str:"foo" ~sub:"bar" ~by:"" = (false, "foo")
@@ -639,27 +645,55 @@ let replace ~str ~sub ~by =
 
 
 let nreplace ~str ~sub ~by =
-  let parts = nsplit str ~by:sub in
-  String.concat by parts
+  if sub = "" then invalid_arg "nreplace: cannot replace all empty substrings" ;
+  let strlen = length str in
+  let sublen = length sub in
+  let bylen = length by in
+  let dlen = bylen - sublen in
+  let rec loop_subst idxes newlen i =
+    match (try rfind_from str (i-1) sub with Not_found -> -1) with
+    | -1 -> idxes, newlen
+    | i' -> loop_subst (i'::idxes) (newlen+dlen) i' in
+  let idxes, newlen = loop_subst [] strlen strlen in
+  let newstr = create newlen in
+  let rec loop_copy i j idxes =
+    match idxes with
+    | [] ->
+      (* still need the last chunk *)
+      unsafe_blit str i newstr j (strlen-i)
+    | i'::rest ->
+      let di = i' - i in
+      unsafe_blit str i newstr j di ;
+      unsafe_blit by 0 newstr (j + di) bylen ;
+      loop_copy (i + di + sublen) (j + di + bylen) rest in
+  loop_copy 0 0 idxes ;
+  newstr
 (*$T nreplace
    nreplace ~str:"bar foo aaa bar" ~sub:"aa" ~by:"foo" = "bar foo afoo bar"
    nreplace ~str:"bar foo bar" ~sub:"bar" ~by:"foo" = "foo foo foo"
+   nreplace ~str:"aaaaaa" ~sub:"aa" ~by:"aaa" = "aaaaaaaaa"
+   nreplace ~str:"" ~sub:"aa" ~by:"bb" = ""
+   nreplace ~str:"foo bar baz" ~sub:"foo bar baz" ~by:"" = ""
+   nreplace ~str:"abc" ~sub:"abc" ~by:"def" = "def"
+   let s1 = "foo" in let s2 = nreplace ~str:s1 ~sub:"X" ~by:"X" in set s2 0 'F' ; s1.[0] = 'f'
 *)
 
 
-let in_place_mirror s =
+let rev_in_place s =
   let len = String.length s in
   if len > 0 then for k = 0 to (len - 1)/2 do
-    let old = s.[k] and mirror = len - 1 - k in
-    s.[k] <- s.[mirror]; s.[mirror] <- old;
-  done
-(*$= in_place_mirror as f & ~printer:identity
+      let old = s.[k] and mirror = len - 1 - k in
+      s.[k] <- s.[mirror]; s.[mirror] <- old;
+    done
+(*$= rev_in_place as f & ~printer:identity
   (let s="" in f s; s)          ""
   (let s="1" in f s; s)         "1"
   (let s="12" in f s; s)        "21"
   (let s="Example!" in f s; s)  "!elpmaxE"
 *)
 
+let in_place_mirror = rev_in_place
+
 let repeat s n =
   let buf = Buffer.create ( n * (String.length s) ) in
   for i = 1 to n do Buffer.add_string buf s done;
@@ -670,6 +704,20 @@ let repeat s n =
    repeat "" 4 = ""
 *)
 
+let rev s = 
+  let len = String.length s in
+  let reversed = String.create len in
+  for i = 0 to len - 1 do
+    String.unsafe_set reversed (len - i - 1) (String.unsafe_get s i)
+  done;
+  reversed
+
+(*$T rev
+   rev "" = ""
+   rev "batteries" = "seirettab"
+   rev "even" = "neve"
+*)
+
 let trim s =
   let len = length s in
   let rec aux_1 i = (*locate leading whitespaces*)
@@ -856,112 +904,112 @@ end (* String.Exceptionless *)
 
 module Cap =
 struct
-type 'a t = string
+  type 'a t = string
 
-let make          = make
-let is_empty      = is_empty
-let init          = init
-let enum          = enum
-let of_enum       = of_enum
-let backwards     = backwards
-let of_backwards  = of_backwards
+  let make          = make
+  let is_empty      = is_empty
+  let init          = init
+  let enum          = enum
+  let of_enum       = of_enum
+  let backwards     = backwards
+  let of_backwards  = of_backwards
 
-let of_int        = of_int
-let of_float      = of_float
-let of_char       = of_char
-let to_int        = to_int
-let to_float      = to_float
-let map           = map
-let fold_left     = fold_left
-let fold_right    = fold_right
-let iter          = iter
-let index         = index
-let rindex        = rindex
-let index_from    = index_from
-let rindex_from   = rindex_from
-let contains      = contains
-let contains_from = contains_from
-let rcontains_from= rcontains_from
-let find          = find
-let find_from     = find_from
-let rfind         = rfind
-let rfind_from    = rfind_from
-let ends_with     = ends_with
-let starts_with   = starts_with
-let exists        = exists
-let lchop         = lchop
-let rchop         = rchop
-let strip         = strip
-let uppercase     = uppercase
-let lowercase     = lowercase
-let capitalize    = capitalize
-let uncapitalize  = uncapitalize
-let copy          = copy
-let sub           = sub
-let fill          = fill
-let blit          = blit
-let concat        = concat
-let escaped       = escaped
-let replace_chars = replace_chars
-let replace       = replace
-let nreplace      = nreplace
-let split         = split
-let repeat        = repeat
-let rsplit        = rsplit
-let nsplit        = nsplit
-let join          = join
-let slice         = slice
-let explode       = explode
-let implode       = implode
-let compare       = compare
-let icompare      = icompare
-let splice        = splice
-let trim          = trim
-let quote         = quote
-let left          = left
-let right         = right
-let head          = head
-let tail          = tail
-let filter_map    = filter_map
-let filter        = filter
-let of_list       = of_list
-let to_list       = to_list
+  let of_int        = of_int
+  let of_float      = of_float
+  let of_char       = of_char
+  let to_int        = to_int
+  let to_float      = to_float
+  let map           = map
+  let fold_left     = fold_left
+  let fold_right    = fold_right
+  let iter          = iter
+  let index         = index
+  let rindex        = rindex
+  let index_from    = index_from
+  let rindex_from   = rindex_from
+  let contains      = contains
+  let contains_from = contains_from
+  let rcontains_from= rcontains_from
+  let find          = find
+  let find_from     = find_from
+  let rfind         = rfind
+  let rfind_from    = rfind_from
+  let ends_with     = ends_with
+  let starts_with   = starts_with
+  let exists        = exists
+  let lchop         = lchop
+  let rchop         = rchop
+  let strip         = strip
+  let uppercase     = uppercase
+  let lowercase     = lowercase
+  let capitalize    = capitalize
+  let uncapitalize  = uncapitalize
+  let copy          = copy
+  let sub           = sub
+  let fill          = fill
+  let blit          = blit
+  let concat        = concat
+  let escaped       = escaped
+  let replace_chars = replace_chars
+  let replace       = replace
+  let nreplace      = nreplace
+  let split         = split
+  let repeat        = repeat
+  let rsplit        = rsplit
+  let nsplit        = nsplit
+  let join          = join
+  let slice         = slice
+  let explode       = explode
+  let implode       = implode
+  let compare       = compare
+  let icompare      = icompare
+  let splice        = splice
+  let trim          = trim
+  let quote         = quote
+  let left          = left
+  let right         = right
+  let head          = head
+  let tail          = tail
+  let filter_map    = filter_map
+  let filter        = filter
+  let of_list       = of_list
+  let to_list       = to_list
 
-let quote         = quote
-let print         = print
-let println       = println
-let print_quoted  = print_quoted
+  let quote         = quote
+  let print         = print
+  let println       = println
+  let print_quoted  = print_quoted
 
-external of_string : string -> _ t                = "%identity"
-external to_string : [`Read | `Write] t -> string = "%identity"
-external read_only : [> `Read] t -> [`Read] t     = "%identity"
-external write_only: [> `Write] t -> [`Write] t   = "%identity"
+  external of_string : string -> _ t                = "%identity"
+  external to_string : [`Read | `Write] t -> string = "%identity"
+  external read_only : [> `Read] t -> [`Read] t     = "%identity"
+  external write_only: [> `Write] t -> [`Write] t   = "%identity"
 
-external length : _ t -> int = "%string_length"
-external get : [> `Read] t -> int -> char = "%string_safe_get"
-external set : [> `Write] t -> int -> char -> unit = "%string_safe_set"
-external create : int -> _ t = "caml_create_string"
-external unsafe_get : [> `Read] t -> int -> char = "%string_unsafe_get"
-external unsafe_set : [> `Write] t -> int -> char -> unit = "%string_unsafe_set"
-external unsafe_blit :
-  [> `Read] t -> int -> [> `Write] t -> int -> int -> unit = "caml_blit_string" "noalloc"
-external unsafe_fill :
-  [> `Write] t -> int -> int -> char -> unit = "caml_fill_string" "noalloc"
+  external length : _ t -> int = "%string_length"
+  external get : [> `Read] t -> int -> char = "%string_safe_get"
+  external set : [> `Write] t -> int -> char -> unit = "%string_safe_set"
+  external create : int -> _ t = "caml_create_string"
+  external unsafe_get : [> `Read] t -> int -> char = "%string_unsafe_get"
+  external unsafe_set : [> `Write] t -> int -> char -> unit = "%string_unsafe_set"
+  external unsafe_blit :
+    [> `Read] t -> int -> [> `Write] t -> int -> int -> unit = "caml_blit_string" "noalloc"
+  external unsafe_fill :
+    [> `Write] t -> int -> int -> char -> unit = "caml_fill_string" "noalloc"
 
-module Exceptionless =
-struct
-  let find_from = Exceptionless.find_from
-  let find = Exceptionless.find
-  let rfind_from = Exceptionless.rfind_from
-  let rfind = Exceptionless.rfind
-  let to_int = Exceptionless.to_int
-  let to_float = Exceptionless.to_float
-  let index = Exceptionless.index
-  let index_from = Exceptionless.index_from
-  let rindex_from = Exceptionless.rindex_from
-  let rindex = Exceptionless.rindex
-  let split = Exceptionless.split
-  let rsplit = Exceptionless.rsplit
-end (* String.Cap.Exceptionless *)
+  module Exceptionless =
+  struct
+    let find_from = Exceptionless.find_from
+    let find = Exceptionless.find
+    let rfind_from = Exceptionless.rfind_from
+    let rfind = Exceptionless.rfind
+    let to_int = Exceptionless.to_int
+    let to_float = Exceptionless.to_float
+    let index = Exceptionless.index
+    let index_from = Exceptionless.index_from
+    let rindex_from = Exceptionless.rindex_from
+    let rindex = Exceptionless.rindex
+    let split = Exceptionless.split
+    let rsplit = Exceptionless.rsplit
+  end (* String.Cap.Exceptionless *)
 
 end (* String.Cap *)
